@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Flask Quorum
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2020 Hive Solutions Lda.
 #
 # This file is part of Hive Flask Quorum.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2020 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -43,9 +43,11 @@ import json
 import time
 import flask
 import atexit
+import socket
 import logging
 import inspect
 import datetime
+import functools
 
 import werkzeug.debug
 
@@ -82,6 +84,37 @@ RUN_F = {}
 """ The map that will contain the various functions that
 will be called upon the start of the main run loop """
 
+ALLOW_ORIGIN = "*"
+""" The default value to be used in the "Access-Control-Allow-Origin"
+header value, this should not be too restrictive """
+
+ALLOW_HEADERS = "*, X-Requested-With"
+""" The default value to be used in the "Access-Control-Allow-Headers"
+header value, this should not be too restrictive """
+
+ALLOW_METHODS = "GET, POST, PUT, DELETE, PATCH, HEAD, OPTIONS"
+""" The default value to be used in the "Access-Control-Allow-Methods"
+header value, this should not be too restrictive """
+
+CONTENT_SECURITY = "default-src * ws://* wss://* data: blob:; script-src * 'unsafe-inline' 'unsafe-eval'; style-src * 'unsafe-inline';"
+""" The default value to be used in the "Content-Security-Policy"
+header value, this should not be too restrictive """
+
+FRAME_OPTIONS = "SAMEORIGIN"
+""" The value to be as the default/original for the "X-Frame-Options"
+header, this should ensure that the same origin is always used when
+trying to embed a dynamic content into a web page """
+
+XSS_PROTECTION = "1; mode=block"
+""" Value to be used as the original one for the "X-XSS-Protection"
+header value, should provide a way of preventing XSS attach under the
+internet explorer browser """
+
+CONTENT_OPTIONS = "nosniff"
+""" Default "X-Content-Type-Options" header value to be used to prevent
+the sniffing of content type values, ensuring that the browser sticks to
+value of content type provided by the server """
+
 ESCAPE_EXTENSIONS = (
     ".xml",
     ".html",
@@ -156,8 +189,8 @@ def prepare_app():
     object used instead of the global one.
 
     :rtype: Application
-    :return: The decorated application object with the proper
-    capabilities enabled, this object should be used for the
+    :return: The decorated application object with the proper\
+    capabilities enabled, this object should be used for the\
     serving operations instead of the global one.
     """
 
@@ -180,7 +213,7 @@ def run_base():
 
 def run_netius():
     try: import netius.servers
-    except BaseException as exception:
+    except Exception as exception:
         raise exceptions.ServerInitError(
             legacy.UNICODE(exception),
             server = "netius"
@@ -220,7 +253,7 @@ def run_netius():
 
 def run_waitress():
     try: import waitress
-    except BaseException as exception:
+    except Exception as exception:
         raise exceptions.ServerInitError(
             legacy.UNICODE(exception),
             server = "waitress"
@@ -285,7 +318,7 @@ def load(
     should be managed using a server side session with support from redis.
     :type mongo_database: String
     :param mongo_database: The default name of the database to be used when\
-    using the mongo infra-structure.
+    using the MongoDB infra-structure.
     :type logger: String
     :param logger: The name to be used as identification for possible logger\
     files created by the logging sub-module.
@@ -328,6 +361,7 @@ def load(
     pusher_app_id = config.conf("PUSHER_APP_ID", None)
     pusher_key = config.conf("PUSHER_KEY", None)
     pusher_secret = config.conf("PUSHER_SECRET", None)
+    pusher_cluster = config.conf("PUSHER_CLUSTER", None)
     smtp_host = config.conf("SMTP_HOST", None)
     smtp_user = config.conf("SMTP_USER", None)
     smtp_password = config.conf("SMTP_PASSWORD", None)
@@ -337,6 +371,33 @@ def load(
     amqp_url = config.conf("CLOUDAMQP_URL", amqp_url)
     amqp_url = config.conf("RABBITMQ_URL", amqp_url)
 
+    # retrieves the possible base URL configuration value and uses it
+    # as the basis for the creation of the static URL values to be used
+    # by the Flask infra-structure when using _external parameter
+    base_url = config.conf("BASE_URL", None)
+    if base_url:
+        base_parse = legacy.urlparse(base_url)
+        if base_parse.port: server_name = "%s:%d" % (base_parse.hostname, base_parse.port)
+        else: server_name = "%s" % base_parse.hostname
+        url_scheme = base_parse.scheme
+        application_root = base_parse.path
+    else:
+        server_name, url_scheme, application_root = None, None, None
+
+    # tries to retrieve the more specific URL related configuration values
+    # defaulting to the base URL calculated ones in case they do not exist
+    server_name = config.conf("SERVER_NAME", server_name)
+    application_root = config.conf("APPLICATION_ROOT", application_root)
+    url_scheme = config.conf("URL_SCHEME", url_scheme)
+    url_scheme = config.conf("PREFERRED_URL_SCHEME", url_scheme)
+
+    # re-sets the URL related configuration values, making sure that they
+    # are properly set in the current configuration environment as they
+    # are critical for proper external URL value generation
+    config.confs("SERVER_NAME", server_name)
+    config.confs("APPLICATION_ROOT", application_root)
+    config.confs("PREFERRED_URL_SCHEME", url_scheme)
+
     # creates the proper values according to the currently provided
     # ones so that they match the ones that are expected
     name = name + "-" + instance if instance else name
@@ -361,9 +422,11 @@ def load(
     APP = app
 
     # loads the app configuration from the provided keyword arguments
-    # map and then starts the logging process with the requested logger
-    # and with the provided "verbosity" level
+    # map and from the current dictionary of configuration values
+    # (value propagation) and then starts the logging process with the
+    # requested logger and provided "verbosity" level
     load_app_config(app, kwargs)
+    load_app_config(app, config.confd())
     start_log(app, name = logger, level = level)
 
     # loads the various paths associated with the application into the
@@ -387,7 +450,9 @@ def load(
     # context filter flag to true the jinja infra-structure will handle
     # the rest of the operations so that it's properly used
     util.nl_to_br_jinja.evalcontextfilter = True
+    util.nl_to_br_jinja.environmentfilter = True
     util.sp_to_nbsp_jinja.evalcontextfilter = True
+    util.sp_to_nbsp_jinja.environmentfilter = True
 
     # updates the base app reference object with a series of operations
     # that should transform it's base execution and behavior
@@ -397,12 +462,21 @@ def load(
     app.template_filter("locale")(util.to_locale)
     app.template_filter("nl_to_br")(util.nl_to_br_jinja)
     app.template_filter("sp_to_nbsp")(util.sp_to_nbsp_jinja)
+    app.template_filter("unset")(util.unset)
     app.request_class = request.Request
     app.locales = locales
     app.safe = safe
     app.debug = debug
     app.use_debugger = debug
     app.use_reloader = reloader
+    app.secure_headers = True
+    app.allow_origin = ALLOW_ORIGIN
+    app.allow_headers = ALLOW_HEADERS
+    app.allow_methods = ALLOW_METHODS
+    app.content_security = CONTENT_SECURITY
+    app.frame_options = FRAME_OPTIONS
+    app.xss_protection = XSS_PROTECTION
+    app.content_options = CONTENT_OPTIONS
     app.models = models
     app.module = module
     app.path = path
@@ -425,6 +499,7 @@ def load(
     if pusher_app_id: pusherc.app_id = pusher_app_id
     if pusher_key: pusherc.key = pusher_key
     if pusher_secret: pusherc.secret = pusher_secret
+    if pusher_cluster: pusherc.cluster = pusher_cluster
     if smtp_host: mail.SMTP_HOST = smtp_host
     if smtp_user: mail.SMTP_USER = smtp_user
     if smtp_password: mail.SMTP_PASSWORD = smtp_password
@@ -456,7 +531,7 @@ def unload():
 
     .. note::
 
-        After the call to this method most of the functionally of quorum\
+        After the call to this method most of the functionally of quorum
         will become unavailable until further call to :func:`quorum.load`.
     """
 
@@ -508,18 +583,18 @@ def load_bundles(app, offset = 2):
     for path in paths:
         # joins the current (base) bundles path with the current path
         # in iteration to create the full path to the file and opens
-        # it trying to read its json based contents
+        # it trying to read its JSON based contents
         path_f = os.path.join(bundles_path, path)
         file = open(path_f, "rb")
         try: data_j = json.load(file)
-        except: continue
+        except Exception: continue
         finally: file.close()
 
         # unpacks the current path in iteration into the base name,
         # locale string and file extension to be used in the registration
         # of the data in the bundles registry
         try: _base, locale, _extension = path.split(".", 2)
-        except: continue
+        except Exception: continue
 
         # retrieves a possible existing map for the current locale in the
         # registry and updates such map with the loaded data, then re-updates
@@ -538,6 +613,20 @@ def start_log(
     # tries to retrieve some of the default configuration values
     # that are going to be used in the logger startup
     format = config.conf("LOGGING_FORMAT", None)
+    file_log = config.conf("FILE_LOG", False, cast = bool)
+    stream_log = config.conf("STREAM_LOG", True, cast = bool)
+    memory_log = config.conf("MEMORY_LOG", True, cast = bool)
+    syslog_host = config.conf("SYSLOG_HOST", None)
+    syslog_port = config.conf("SYSLOG_PORT", None, cast = int)
+    syslog_proto = config.conf("SYSLOG_PROTO", "udp")
+    syslog_kwargs = dict(socktype = socket.SOCK_STREAM) if\
+        syslog_proto in ("tcp",) else dict()
+    syslog_log = True if syslog_host else False
+
+    # tries to determine the default syslog port in case no port
+    # is defined and syslog logging is enabled
+    if not syslog_port and syslog_log:
+        syslog_port = log.SYSLOG_PORTS.get(syslog_proto)
 
     # "resolves" the proper logger file path taking into account
     # the currently defined operative system, should uses the system
@@ -567,14 +656,14 @@ def start_log(
 
     # creates both the stream and the memory based handlers that
     # are going to be used for the current logger
-    stream_handler = logging.StreamHandler()
-    memory_handler = log.MemoryHandler()
+    stream_handler = logging.StreamHandler() if stream_log else None
+    memory_handler = log.MemoryHandler() if memory_log else None
 
     try:
         # tries to create the file handler for the logger with the
         # resolve path (operation may fail for permissions)
-        file_handler = path and logging.FileHandler(path)
-    except:
+        file_handler = path and file_log and logging.FileHandler(path)
+    except Exception:
         # in case there's an error creating the file handler for
         # the logger prints an error message indicating the problem
         sys.stderr.write("Problem starting logging for file '%s'\n" % path)
@@ -599,6 +688,25 @@ def start_log(
         handler.setFormatter(formatter)
         handler.setLevel(level)
 
+    # determines if the creation of the syslog handler is required and
+    # it that the case created it setting the appropriate formatter to it
+    syslog_handler = logging.handlers.SysLogHandler(
+        (syslog_host, syslog_port), **syslog_kwargs
+    ) if syslog_log else None if syslog_log else None
+
+    # in case the syslog handler has been created creates the appropriate
+    # formatter for it, sets the level and adds it to the logger
+    if syslog_handler:
+        syslog_formatter = log.BaseFormatter(
+            log.LOGGIGN_SYSLOG % APP.name if APP else "quorum",
+            datefmt = "%Y-%m-%dT%H:%M:%S.000000+00:00",
+            wrap = True
+        )
+        syslog_handler.setLevel(level)
+        syslog_handler.setFormatter(syslog_formatter)
+        logger.addHandler(syslog_handler)
+        app.handlers["syslog"] = syslog_handler
+
     # runs the extra logging step for the current state, meaning that
     # some more handlers may be created according to the logging config
     extra_logging(logger, level, formatter)
@@ -687,17 +795,22 @@ def get_handler(name, app = None):
     if not app: return None
     return app.handlers.get(name, None)
 
-def get_bundle(name, app = None):
+def get_bundle(name, app = None, split = True):
     app = app or APP
     if not app: return None
     bundle = app.bundles.get(name, None)
     if bundle: return bundle
+    if split and name:
+        base = name.split("_", 1)[0]
+        bundle = app.bundles.get(base, None)
+        if bundle: return bundle
     name = _best_locale(name)
     return app.bundles.get(name, None)
 
 def is_devel(app = None):
     level = get_level(app = app)
     if not level: return False
+    return level < logging.INFO
 
 def finalize(value):
     # returns an empty string as value representation
@@ -715,6 +828,7 @@ def before_request():
 def after_request(response):
     if APP.safe: util.reset_locale()
     util.anotate_async(response)
+    util.anotate_secure(response)
     return response
 
 def context_processor():
@@ -773,7 +887,7 @@ def models_c(models = None):
         # from the top level model in case it does not continues
         # the loop as there's nothing to be done
         try: is_valid = issubclass(value, model.Model)
-        except: is_valid = False
+        except Exception: is_valid = False
         if not is_valid: continue
 
         # adds the current value in iteration as a new class
@@ -818,7 +932,37 @@ def base_path(*args, **kwargs):
     return os.path.join(APP.root_path, *args)
 
 def has_context():
-    return flask._app_ctx_stack.top
+    return True if flask._app_ctx_stack.top else False
+
+def ensure_context(function):
+    """
+    Decorator that makes sure that the underlying execution
+    method/function is run inside a valid app context.
+
+    In case there's currently no app context defined it uses
+    the global Application reference to create a new one.
+
+    :type function: Function
+    :param function: The function that is going to be decorated
+    by the current ensure decorator.
+    :rtype: Decorator
+    :return: The decorator that should be used for the ensuring
+    of the app context in the current execution environment.
+    """
+
+    @functools.wraps(function)
+    def interceptor(*args, **kwargs):
+        _ctx = has_context()
+        _app_ctx = APP.app_context() if APP else None
+        _ensure = True if not _ctx and _app_ctx else False
+        try:
+            if _ensure: flask._app_ctx_stack.push(_app_ctx)
+            result = function(*args, **kwargs)
+        finally:
+            if _ensure: flask._app_ctx_stack.pop()
+        return result
+
+    return interceptor
 
 def onrun(function):
     fname = function.__name__
