@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Flask Quorum
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2020 Hive Solutions Lda.
 #
 # This file is part of Hive Flask Quorum.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2020 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -47,7 +47,7 @@ from . import log
 
 BACKGROUND = []
 """ The list containing the various global registered
-function to be executed as background operations, note
+functions to be executed as background operations, note
 that only the name is used in the list so a possible
 collision of tasks is possible """
 
@@ -87,7 +87,7 @@ class ExecutionThread(threading.Thread):
         Constructor of the class.
         """
 
-        threading.Thread.__init__(self)
+        threading.Thread.__init__(self, name = "Execution")
 
         self.daemon = True
         self.work_list = []
@@ -143,10 +143,11 @@ class ExecutionThread(threading.Thread):
                 # executes the "callable" and logs the error in case the
                 # execution fails (must be done to log the error) then
                 # sets the error flag with the exception variable
-                try: callable(*args, **kwargs)
-                except BaseException as exception:
+                try:
+                    callable(*args, **kwargs)
+                except Exception as exception:
                     error = exception
-                    log.warning(str(exception) + "\n")
+                    log.warning(str(exception), log_trace = True)
 
                 # calls the callback method with the currently set error
                 # in order to notify the runtime about the problem, only
@@ -258,6 +259,14 @@ def seconds_work(callable, offset = 0, *args, **kwargs):
     eval = lambda: seconds_eval(offset)
     return interval_work(callable, eval = eval, *args, **kwargs)
 
+def minutes_work(callable, offset = 0, *args, **kwargs):
+    eval = lambda: minutes_eval(offset)
+    return interval_work(callable, eval = eval, *args, **kwargs)
+
+def hourly_work(callable, offset = 0, *args, **kwargs):
+    eval = lambda: hourly_eval(offset)
+    return interval_work(callable, eval = eval, *args, **kwargs)
+
 def daily_work(callable, offset = 0, *args, **kwargs):
     eval = lambda: daily_eval(offset)
     return interval_work(callable, eval = eval, *args, **kwargs)
@@ -276,6 +285,26 @@ def seconds_eval(offset, now = None):
     next_tuple = next.utctimetuple()
     return calendar.timegm(next_tuple)
 
+def minutes_eval(offset, now = None):
+    now = now or datetime.datetime.utcnow()
+    current = datetime.datetime(
+        year = now.year,
+        month = now.month,
+        day = now.day,
+        hour = now.hour,
+        minute = now.minute
+    )
+    next = current + datetime.timedelta(minutes = 1, seconds = offset)
+    next_tuple = next.utctimetuple()
+    return calendar.timegm(next_tuple)
+
+def hourly_eval(offset, now = None):
+    now = now or datetime.datetime.utcnow()
+    current = datetime.datetime(year = now.year, month = now.month, day = now.day, hour = now.hour)
+    next = current + datetime.timedelta(hours = 1, seconds = offset)
+    next_tuple = next.utctimetuple()
+    return calendar.timegm(next_tuple)
+
 def daily_eval(offset, now = None):
     now = now or datetime.datetime.utcnow()
     today = datetime.datetime(year = now.year, month = now.month, day = now.day)
