@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Flask Quorum
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2020 Hive Solutions Lda.
 #
 # This file is part of Hive Flask Quorum.
 #
@@ -31,27 +31,30 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2020 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
 """ The license for the module """
 
+import os
 import json
 import base64
 import random
 import string
 import logging
 
+from . import util
 from . import legacy
 from . import typesf
 from . import config
 from . import common
 from . import exceptions
+from . import structures
 
 TIMEOUT = 60
 """ The timeout in seconds to be used for the blocking
-operations in the http connection """
+operations in the HTTP connection """
 
 RANGE = string.ascii_letters + string.digits
 """ The range of characters that are going to be used in
@@ -62,14 +65,20 @@ SEQUENCE_TYPES = (list, tuple)
 considered to be sequence based for python """
 
 AUTH_ERRORS = (401, 403, 440, 499)
-""" The sequence that defines the various http errors
+""" The sequence that defines the various HTTP errors
 considered to be authentication related and for which a
 new authentication try will be performed """
 
-def try_auth(auth_callback, params, headers = None):
-    if not auth_callback: raise
-    if headers == None: headers = dict()
-    auth_callback(params, headers)
+def file_g(path, chunk = 40960):
+    yield os.path.getsize(path)
+    file = open(path, "rb")
+    try:
+        while True:
+            data = file.read(chunk)
+            if not data: break
+            yield data
+    finally:
+        file.close()
 
 def get_f(*args, **kwargs):
     name = kwargs.pop("name", "default")
@@ -91,7 +100,7 @@ def get(url, auth_callback = None, **kwargs):
             return _get(url, **kwargs)
         except legacy.HTTPError as error:
             if error.code in AUTH_ERRORS and auth_callback:
-                try_auth(auth_callback, kwargs)
+                _try_auth(auth_callback, kwargs)
             else:
                 code = error.getcode()
                 raise exceptions.HTTPDataError(error, code)
@@ -129,7 +138,7 @@ def get_json(
             )
         except legacy.HTTPError as error:
             if error.code in AUTH_ERRORS and auth_callback:
-                try_auth(auth_callback, kwargs)
+                _try_auth(auth_callback, kwargs)
             else:
                 data_r = error.read()
                 data_r = legacy.str(data_r, encoding = "utf-8")
@@ -177,7 +186,7 @@ def post_json(
             )
         except legacy.HTTPError as error:
             if error.code in AUTH_ERRORS and auth_callback:
-                try_auth(auth_callback, kwargs)
+                _try_auth(auth_callback, kwargs)
             else:
                 data_r = error.read()
                 data_r = legacy.str(data_r, encoding = "utf-8")
@@ -225,7 +234,7 @@ def put_json(
             )
         except legacy.HTTPError as error:
             if error.code in AUTH_ERRORS and auth_callback:
-                try_auth(auth_callback, kwargs)
+                _try_auth(auth_callback, kwargs)
             else:
                 data_r = error.read()
                 data_r = legacy.str(data_r, encoding = "utf-8")
@@ -265,7 +274,7 @@ def delete_json(
             )
         except legacy.HTTPError as error:
             if error.code in AUTH_ERRORS and auth_callback:
-                try_auth(auth_callback, kwargs)
+                _try_auth(auth_callback, kwargs)
             else:
                 data_r = error.read()
                 data_r = legacy.str(data_r, encoding = "utf-8")
@@ -278,6 +287,11 @@ def delete_json(
         if retries == 0:
             raise exceptions.HTTPError("Data retrieval not possible")
 
+def _try_auth(auth_callback, params, headers = None):
+    if not auth_callback: raise
+    if headers == None: headers = dict()
+    auth_callback(params, headers)
+
 def _get(url, **kwargs):
     values = kwargs or dict()
     data = _urlencode(values)
@@ -419,6 +433,7 @@ def _method_empty(
         redirect = redirect,
         timeout = timeout
     )
+    result = _result(result, info)
     return (result, file) if handle else result
 
 def _method_payload(
@@ -461,11 +476,14 @@ def _method_payload(
         data = data_e
         mime = mime or "application/x-www-form-urlencoded"
 
-    data = legacy.bytes(data)
-    length = len(data) if data else 0
+    if legacy.is_unicode(data): data = legacy.bytes(data, force = True)
+
+    if not data: length = 0
+    elif legacy.is_bytes(data): length = len(data)
+    else: length = -1
 
     headers = dict(headers) if headers else dict()
-    headers["Content-Length"] = length
+    if not length == -1: headers["Content-Length"] = length
     if mime: headers["Content-Type"] = mime
     if host: headers["Host"] = host
     if authorization: headers["Authorization"] = "Basic %s" % authorization
@@ -488,6 +506,7 @@ def _method_payload(
         timeout = timeout
     )
 
+    result = _result(result, info)
     return (result, file) if handle else result
 
 def _redirect(
@@ -533,6 +552,10 @@ def _resolve(*args, **kwargs):
     return result
 
 def _resolve_legacy(url, method, headers, data, silent, timeout, **kwargs):
+    is_generator = not data == None and legacy.is_generator(data)
+    if is_generator: next(data); data = b"".join(data)
+    is_file = hasattr(data, "tell")
+    if is_file: data = data.read()
     opener = legacy.build_opener(legacy.HTTPHandler)
     request = legacy.Request(url, data = data, headers = headers)
     request.get_method = lambda: method
@@ -540,6 +563,15 @@ def _resolve_legacy(url, method, headers, data, silent, timeout, **kwargs):
 
 def _resolve_requests(url, method, headers, data, silent, timeout, **kwargs):
     import requests
+
+    # verifies if the provided data is a generator, assumes that if the
+    # data is not invalid and is of type generator then it's a generator
+    # and then if that's the case encapsulates this size based generator
+    # into a generator based file-like object so that it can be used inside
+    # the request infra-structure (as it accepts only file objects)
+    is_generator = not data == None and legacy.is_generator(data)
+    if is_generator: data = structures.GeneratorFile(data)
+
     method = method.lower()
     caller = getattr(requests, method)
     result = caller(url, headers = headers, data = data, timeout = timeout)
@@ -566,7 +598,7 @@ def _resolve_netius(url, method, headers, data, silent, timeout, **kwargs):
         url,
         headers = headers,
         data = data,
-        async = False,
+        asynchronous = False,
         timeout = timeout,
         level = level
     )
@@ -595,17 +627,20 @@ def _parse_url(url):
 
 def _result(data, info = {}, force = False, strict = False):
     # tries to retrieve the content type value from the headers
-    # info and verifies if the current data is json encoded, so
+    # info and verifies if the current data is JSON encoded, so
     # that it gets automatically decoded for such cases
     content_type = info.get("Content-Type", None) or ""
-    is_json = content_type.startswith((
-        "application/json",
-        "text/json",
-        "text/javascript"
-    )) or force
+    is_json = util.is_content_type(
+        content_type,
+        (
+            "application/json",
+            "text/json",
+            "text/javascript"
+        )
+    ) or force
 
-    # verifies if the current result set is json encoded and in
-    # case it's decodes it and loads it as json otherwise returns
+    # verifies if the current result set is JSON encoded and in
+    # case it's decodes it and loads it as JSON otherwise returns
     # the "raw" data to the caller method as expected, note that
     # the strict flag is used to determine if the exception should
     # be re-raised to the upper level in case of value error
@@ -645,14 +680,18 @@ def _params(query):
     return params
 
 def _urlencode(values, as_string = True):
-    # creates the dictionary that will hold the final
-    # dictionary of values (without the unset andÂ´
-    # invalid values)
-    final = dict()
+    # creates the list that will hold the final tuple of values
+    # (without the unset and invalid values)
+    final = []
 
-    # iterates over all the items in the values map to
+    # verifies if the provided value is a sequence and in case it's
+    # not converts it into a sequence (assuming a map)
+    is_sequence = isinstance(values, (list, tuple))
+    if not is_sequence: values = values.items()
+
+    # iterates over all the items in the values sequence to
     # try to filter the values that are not valid
-    for key, value in values.items():
+    for key, value in values:
         # creates the list that will hold the valid values
         # of the current key in iteration (sanitized values)
         _values = []
@@ -684,7 +723,7 @@ def _urlencode(values, as_string = True):
 
         # sets the sanitized list of values as the new value for
         # the key in the final dictionary of values
-        final[key] = _values
+        final.append((key, _values))
 
     # in case the "as string" flag is not set the ended key to value
     # dictionary should be returned to the called method and not the
@@ -730,6 +769,7 @@ def _encode_multipart(fields, mime = None, doseq = False):
         values = values if is_list else [values]
 
         for value in values:
+            if value == None: continue
 
             if isinstance(value, dict):
                 header_l = []
@@ -784,7 +824,9 @@ def _try_boundary(fields, boundary, doseq = False):
 
         for value in values:
             if isinstance(value, dict): name = ""; value = value.get("data", b"")
-            elif isinstance(value, tuple): name = value[0] or ""; value = value[1]
+            elif isinstance(value, tuple):
+                if len(value) == 2: name = value[0] or ""; value = value[1] or b""
+                else: name = value[0] or ""; value = value[2] or b""
             else: name = ""; value = _encode(value)
 
             if not key.find(boundary) == -1: return False
