@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Flask Quorum
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2020 Hive Solutions Lda.
 #
 # This file is part of Hive Flask Quorum.
 #
@@ -31,18 +31,18 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2020 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
 """ The license for the module """
 
 NAME = "quorum"
-VERSION = "0.5.0"
+VERSION = "0.6.0"
 AUTHOR = "Hive Solutions Lda."
 EMAIL = "development@hive.pt"
 DESCRIPTION = "Quorum Extensions for Flask"
 LICENSE = "Apache License, Version 2.0"
 KEYWORDS = "quorum flask"
-URL = "http://flask_quorum.hive.pt"
-COPYRIGHT = "2008-2017 Hive Solutions Lda."
+URL = "http://flask-quorum.hive.pt"
+COPYRIGHT = "2008-2020 Hive Solutions Lda."
