@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Flask Quorum
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2020 Hive Solutions Lda.
 #
 # This file is part of Hive Flask Quorum.
 #
@@ -31,28 +31,33 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2020 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
 """ The license for the module """
 
 import sys
+import json
+import socket
 import inspect
+import itertools
 import threading
 import traceback
+import collections
 
 import logging.handlers
 
 from . import common
 from . import config
+from . import legacy
 
-LOGGING_FORMAT = "%%(asctime)s [%%(levelname)s] %s%%(message)s"
+LOGGING_FORMAT_T = "%%(asctime)s [%%(levelname)s] %s%%(message)s"
 """ The format to be used for the logging operation in
 the app, these operations are going to be handled by
 multiple stream handlers """
 
-LOGGING_FORMAT_TID = "%%(asctime)s [%%(levelname)s] %s[%%(thread)d] %%(message)s"
+LOGGING_FORMAT_TID_T = "%%(asctime)s [%%(levelname)s] %s[%%(thread)d] %%(message)s"
 """ The format to be used for the logging operation in
 the app, these operations are going to be handled by
 multiple stream handlers, this version of the string
@@ -63,6 +68,11 @@ LOGGING_EXTRA = "[%(name)s] " if config.conf("LOGGING_EXTRA", cast = bool) else
 """ The extra logging attributes that are going to be applied
 to the format strings to obtain the final on the logging """
 
+LOGGIGN_SYSLOG = "1 %%(asctime)s %%(hostname)s %s %%(process)d %%(thread)d \
+[quorumSDID@0 tid=\"%%(thread)d\"] %%(json)s"
+""" The format to be used for the message sent using the syslog
+logger, should contain extra structured data """
+
 MAX_LENGTH = 10000
 """ The maximum amount of messages that are kept in
 memory until they are discarded, avoid a very large
@@ -95,8 +105,12 @@ LEVEL_ALIAS = {
 """ Map defining a series of alias that may be used latter
 for proper debug level resolution """
 
-LOGGING_FORMAT = LOGGING_FORMAT % LOGGING_EXTRA
-LOGGING_FORMAT_TID = LOGGING_FORMAT_TID % LOGGING_EXTRA
+SYSLOG_PORTS = dict(tcp = 601, udp = 514)
+""" Dictionary that maps the multiple transport protocol
+used by syslog with the appropriate default ports """
+
+LOGGING_FORMAT = LOGGING_FORMAT_T % LOGGING_EXTRA
+LOGGING_FORMAT_TID = LOGGING_FORMAT_TID_T % LOGGING_EXTRA
 
 class MemoryHandler(logging.Handler):
     """
@@ -108,8 +122,8 @@ class MemoryHandler(logging.Handler):
     def __init__(self, level = logging.NOTSET, max_length = MAX_LENGTH):
         logging.Handler.__init__(self, level = level)
         self.max_length = max_length
-        self.messages = []
-        self.messages_l = {}
+        self.messages = collections.deque()
+        self.messages_l = dict()
 
         format = config.conf("LOGGING_FORMAT", None)
         format_base = format or LOGGING_FORMAT
@@ -123,21 +137,22 @@ class MemoryHandler(logging.Handler):
         # in case the level is not found in the list of levels
         # it's not considered valid and so an empty list is returned
         try: index = LEVELS.index(level)
-        except: return []
+        except Exception: return collections.deque()
 
         # retrieves the complete set of levels that are considered
-        # equal or less sever than the requested one
+        # equal or more severe than the requested one
         levels = LEVELS[:index + 1]
 
         # creates the list that will hold the various message
         # lists associated with the current severity level
-        messages_l = []
+        messages_l = collections.deque()
 
         # iterates over the complete set of levels considered
         # equal or less sever to add the respective messages
         # list to the list of message lists
         for level in levels:
-            _messages_l = self.messages_l.get(level, [])
+            _messages_l = self.messages_l.get(level, None)
+            if _messages_l == None: _messages_l = collections.deque()
             self.messages_l[level] = _messages_l
             messages_l.append(_messages_l)
 
@@ -161,7 +176,7 @@ class MemoryHandler(logging.Handler):
         # inserts the message into the messages queue and in
         # case the current length of the message queue overflows
         # the one defined as maximum must pop message from queue
-        self.messages.insert(0, message)
+        self.messages.appendleft(message)
         messages_s = len(self.messages)
         if messages_s > self.max_length: self.messages.pop()
 
@@ -171,18 +186,87 @@ class MemoryHandler(logging.Handler):
             # inserts the message into the proper level specific queue
             # and in case it overflows runs the same pop operation as
             # specified also for the more general queue
-            _messages_l.insert(0, message)
+            _messages_l.appendleft(message)
             messages_s = len(_messages_l)
             if messages_s > self.max_length: _messages_l.pop()
 
+    def clear(self):
+        self.messages = collections.deque()
+        self.messages_l = dict()
+
     def get_latest(self, count = None, level = None):
         count = count or 100
+        is_level = level and not legacy.is_string(level)
+        if is_level: level = logging.getLevelName(level)
         level = level.upper() if level else level
         level = LEVEL_ALIAS.get(level, level)
-        messages = self.messages_l.get(level, ()) if level else self.messages
-        return messages[:count]
+        messages = self.messages_l.get(level, []) if level else self.messages
+        slice = itertools.islice(messages, 0, count)
+        return list(slice)
 
-class ThreadFormatter(logging.Formatter):
+    def flush_to_file(
+        self,
+        path,
+        count = None,
+        level = None,
+        reverse = True,
+        clear = True
+    ):
+        messages = self.get_latest(level = level, count = count or 65536)
+        if not messages: return
+        if reverse: messages.reverse()
+        is_path = isinstance(path, legacy.STRINGS)
+        file = open(path, "wb") if is_path else path
+        try:
+            for message in messages:
+                message = legacy.bytes(message, "utf-8", force = True)
+                file.write(message + b"\n")
+        finally:
+            if is_path: file.close()
+        if clear: self.clear()
+
+class BaseFormatter(logging.Formatter):
+    """
+    The base Quorum logging formatted used to add some extra
+    functionality on top of Python's base formatting infra-structure.
+
+    Most of its usage focus on empowering the base record object
+    with some extra values.
+    """
+
+    def __init__(self, *args, **kwargs):
+        self._wrap = kwargs.pop("wrap", False)
+        logging.Formatter.__init__(self, *args, **kwargs)
+
+    @classmethod
+    def _wrap_record(cls, record):
+        if hasattr(record, "_wrapped"): return
+        record.hostname = socket.gethostname()
+        record.json = json.dumps(
+            dict(
+                message = str(record.msg),
+                hostname = record.hostname,
+                lineno = record.lineno,
+                module = record.module,
+                callable = record.funcName,
+                level = record.levelname,
+                thread = record.thread,
+                process = record.process,
+                logger = record.name
+            )
+        )
+        record._wrapped = True
+
+    def format(self, record):
+        # runs the wrapping operation on the record so that more
+        # information becomes available in it (as expected)
+        if self._wrap: self.__class__._wrap_record(record)
+
+        # runs the basic format operation on the record so that
+        # it gets properly formatted into a plain string
+        return logging.Formatter.format(self, record)
+
+class ThreadFormatter(BaseFormatter):
     """
     Custom formatter class that changing the default format
     behavior so that the thread identifier is printed when
@@ -190,20 +274,28 @@ class ThreadFormatter(logging.Formatter):
     """
 
     def __init__(self, *args, **kwargs):
-        logging.Formatter.__init__(self, *args, **kwargs)
-        self._tidfmt = logging.Formatter(self._fmt)
+        BaseFormatter.__init__(self, *args, **kwargs)
+        self._basefmt = BaseFormatter(*args, **kwargs)
+        self._tidfmt = BaseFormatter(*args, **kwargs)
 
     def format(self, record):
+        # runs the wrapping operation on the record so that more
+        # information becomes available in it (as expected)
+        if self._wrap: self.__class__._wrap_record(record)
+
         # retrieves the reference to the current thread and verifies
         # if it represent the current process main thread, then selects
         # the appropriate formating string taking that into account
         current = threading.current_thread()
         is_main = current.name == "MainThread"
         if not is_main: return self._tidfmt.format(record)
-        return logging.Formatter.format(self, record)
+        return self._basefmt.format(record)
 
-    def set_tid(self, value):
-        self._tidfmt = logging.Formatter(value)
+    def set_base(self, value, *args, **kwargs):
+        self._basefmt = BaseFormatter(value, *args, **kwargs)
+
+    def set_tid(self, value, *args, **kwargs):
+        self._tidfmt = BaseFormatter(value, *args, **kwargs)
 
 def rotating_handler(
     path = "quorum.log",
