@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Flask Quorum
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2020 Hive Solutions Lda.
 #
 # This file is part of Hive Flask Quorum.
 #
@@ -31,12 +31,14 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2020 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
 """ The license for the module """
 
+import os
+import uuid
 import flask
 import smtplib
 
@@ -46,6 +48,7 @@ import email.header
 import email.mime.multipart
 import email.mime.text
 
+from . import httpc
 from . import config
 from . import common
 from . import legacy
@@ -73,7 +76,7 @@ def send_mail(
     :rst:dir:`SMTP_USER` and :rst:dir:`SMTP_PASSWORD` configurations.
 
     The email message is sent under the ``alternative`` mime
-    type so that both the plain text and the rich text (html)
+    type so that both the plain text and the rich text (HTML)
     parts are sent in the same message.
 
     The ``plain`` and ``rich`` arguments allow the user to process
@@ -121,19 +124,19 @@ def send_mail(
     :param encoding: The text encoding name that is going to be used in the\
     mail that is going to be sent, should be used with care.
     :type host: String
-    :param host: The hostname for the connection of the smtp server that\
+    :param host: The hostname for the connection of the SMTP server that\
     is going to be used, this may be wither a domain of an address.
     :type port: int
-    :param port: The tcp port number that is going to be used for the client\
+    :param port: The TCP port number that is going to be used for the client\
     connection with the server.
     :type username: String
     :param username: Username value that is used for the authentication part\
-    of the connection with the smtp server.
+    of the connection with the SMTP server.
     :type password: String
     :param password: Secret password to be used as part of the authentication\
-    process of the created smtp connection.
+    process of the created SMTP connection.
     :type stls: bool
-    :param stls: If the connection with the target smtp server should be made\
+    :param stls: If the connection with the target SMTP server should be made\
     using a secure mechanism of a plain text one.
     :type safe: bool
     :param safe: If the current email message should be used using a safe\
@@ -148,39 +151,64 @@ def send_mail(
     # the email is left empty (required by specification)
     data = data or "This part of the email is empty"
 
+    # tries to retrieve the URL based definition of the SMTP
+    # settings so that they may be used for configuration
+    url = config.conf("SMTP_URL", None)
+    url_p = legacy.urlparse(url) if url else None
+    if url_p:
+        host_p, port_p, user_p, password_p, stls_p =\
+        url_p.hostname,\
+        url_p.port,\
+        url_p.username,\
+        url_p.password,\
+        url_p.scheme == "smtps"
+    else:
+        host_p, port_p, user_p, password_p, stls_p =\
+        None, None, None, None, None
+
+    # runs the defaulting operation for host and port definition
+    # that haven't been set (should follow SMTP defaults)
+    if host_p == None: host_p = "localhost"
+    if port_p == None: port_p = 25
+
     # tries to retrieve the various configuration values that are going
-    # to be used for the establishment of the smtp connection, taking
+    # to be used for the establishment of the SMTP connection, taking
     # into account both the provided parameters and the configuration
     # variables currently defined for the environment
-    host = host or config.conf("SMTP_HOST", "localhost")
-    port = port or config.conf("SMTP_PORT", 25, cast = int)
-    username = username or config.conf("SMTP_USER", None)
-    password = password or config.conf("SMTP_PASSWORD", None)
-    stls = password or stls or config.conf("SMTP_STARTTLS", True, cast = int)
+    host = host or config.conf("SMTP_HOST", host_p)
+    port = port or config.conf("SMTP_PORT", port_p, cast = int)
+    username = username or config.conf("SMTP_USER", user_p)
+    password = password or config.conf("SMTP_PASSWORD", password_p)
+    stls = password or stls or config.conf("SMTP_STARTTLS", stls_p, cast = bool)
 
-    # sets the sender with the smtp user value in case no values
+    # sets the sender with the SMTP user value in case no values
     # has been provided (expected behavior)
     sender = sender or username
 
+    # tries to run the template resolution process, by using this approach
+    # it's possible to retrieve remote based templates (dynamic loading)
+    plain = _try_resolve(plain)
+    rich = _try_resolve(rich)
+
     # renders the (possible existing) templates in both the plain
     # and rich text object retrieving the final data to be sent
     plain_data = plain and _render(app, plain, **context) or data
     html_data = rich and _render(app, rich, **context) or data
 
-    # verifies the existence of data (both plain and html) and in
+    # verifies the existence of data (both plain and HTML) and in
     # there's data it should be encoded using the currently provided
     # one so that the raw data is used instead of the unicode one
     if plain_data: plain_data = plain_data.encode(encoding)
     if html_data: html_data = html_data.encode(encoding)
 
     # creates the mime's multipart object with the appropriate header
-    # values set and in the alternative model (for html compatibility)
+    # values set and in the alternative model (for HTML compatibility)
     message = _multipart()
     message["Subject"] = subject
     message["From"] = _format(sender)
     message["To"] = ", ".join(_format(receiver) for receiver in receivers)
 
-    # creates both the plain text and the rich text (html) objects
+    # creates both the plain text and the rich text (HTML) objects
     # from the provided data and then attached them to the message
     # (multipart alternative) that is the base structure
     plain = plain_data and _plain(plain_data, encoding = encoding)
@@ -196,7 +224,7 @@ def send_mail(
         contents = contents.replace("\r\n", "\n")
         contents = contents.replace("\n", "\r\n")
 
-    # creates the connection with the smtp server and starts the tls
+    # creates the connection with the SMTP server and starts the tls
     # connection to send the created email message
     server = smtplib.SMTP(host, port = port)
     try:
@@ -238,6 +266,22 @@ def _format(address, encoding = "utf-8"):
     address_email = str(address_email)
     return "%s <%s>" % (address_name, address_email)
 
+def _try_resolve(template_name):
+    if not template_name: return template_name
+    if not template_name.startswith(("http://", "https://")): return template_name
+    contents = httpc.get(template_name)
+    base_name = os.path.basename(template_name)
+    base_split = base_name.split(".", 1)
+    base_extension = "." + base_split[1] if len(base_split) > 1 else ""
+    file_name = str(uuid.uuid4()) + base_extension
+    file_path = os.path.join(common.base().templates_path(), file_name)
+    if not os.path.exists(common.base().templates_path()):
+        os.makedirs(common.base().templates_path())
+    file = open(file_path, "wb")
+    try: file.write(contents)
+    finally: file.close()
+    return file_name
+
 def _render(app, template_name, **context):
     template = app.jinja_env.get_or_select_template(template_name)
     return flask.templating._render(template, context, app)
