@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Flask Quorum
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2020 Hive Solutions Lda.
 #
 # This file is part of Hive Flask Quorum.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2020 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -41,24 +41,25 @@ import os
 import json
 import shelve
 
+from . import util
 from . import config
 from . import exceptions
 
 try: import redis
-except: redis = None
+except ImportError: redis = None
 
 connection = None
 """ The global connection object that should persist
 the connection relation with the database service """
 
 url = None
-""" The global variable containing the url to be used
+""" The global variable containing the URL to be used
 for the connection with the service """
 
 class RedisMemory(object):
     """
     "Local" in memory stub object that simulates
-    the redis interface, useful for debugging.
+    the Redis interface, useful for debugging.
 
     This memory interface may create problems in
     a multiple process environment (non shared memory).
@@ -66,7 +67,7 @@ class RedisMemory(object):
 
     values = None
     """ The map containing the various values to
-    be set in the memory map, simulates the redis
+    be set in the memory map, simulates the Redis
     data store """
 
     def __init__(self):
@@ -80,7 +81,7 @@ class RedisMemory(object):
         name_s = str(name)
         self.values[name_s] = value
 
-    def setex(self, name, value, expire):
+    def setex(self, name, value, time = None):
         self.set(name, value)
 
     def delete(self, name):
@@ -90,7 +91,7 @@ class RedisMemory(object):
 class RedisShelve(RedisMemory):
     """
     "Local" in persistent stub object that simulates
-    the redis interface, useful for debugging.
+    the Redis interface, useful for debugging.
 
     This shelve interface requires a writable path
     where its persistent file may be written.
@@ -145,5 +146,13 @@ def dumps(*args):
 def _get_connection(url):
     global connection
     if redis == None: raise exceptions.ModuleNotFound("redis")
-    if not connection: connection = url and redis.from_url(url) or RedisShelve()
+    if not connection: connection = url and _redis().from_url(url) or RedisShelve()
     return connection
+
+def _redis(verify = True):
+    if verify: util.verify(
+        not redis == None,
+        message = "RedisPy library not available",
+        exception = exceptions.OperationalError
+    )
+    return redis
