@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Flask Quorum
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2020 Hive Solutions Lda.
 #
 # This file is part of Hive Flask Quorum.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2020 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -50,14 +50,14 @@ class ModelTest(quorum.TestCase):
                 mongo_database = "test",
                 models = mock
             )
-        except:
+        except Exception:
             self.skip()
 
     def tearDown(self):
         try:
             adapter = quorum.get_adapter()
             adapter.drop_db()
-        except: pass
+        except Exception: pass
         finally: quorum.unload()
 
     @quorum.secured
@@ -226,67 +226,6 @@ class ModelTest(quorum.TestCase):
         self.assertEqual(person_m["cats"][0]["identifier_safe"], 1)
         self.assertEqual(person_m["cats"][0]["name"], "NameCat")
 
-    @quorum.secured
-    def test_references(self):
-        person = mock.Person()
-        person.name = "Name"
-
-        cat = mock.Cat()
-        cat.name = "NameCat"
-        cat.save()
-
-        person.cats = [cat]
-        person.save()
-
-        person = mock.Person.get(identifier = 1)
-
-        self.assertEqual(person.cats[0].name, "NameCat")
-
-        person.cats = mock.Person.cats["type"]([cat])
-        person.save()
-
-        person = mock.Person.get(identifier = 1)
-
-        self.assertEqual(person.cats.is_resolved(), False)
-        self.assertEqual(person.car, None)
-        self.assertEqual(person.cats[0].name, "NameCat")
-
-        person = mock.Person.get(identifier = 1, map = True)
-
-        self.assertEqual(isinstance(person, dict), True)
-        self.assertEqual(isinstance(person["cats"], list), True)
-        self.assertEqual(isinstance(person["cats"][0], int), True)
-        self.assertEqual(len(person["cats"]), 1)
-
-        person = mock.Person.get(identifier = 1, eager = ("cats",))
-
-        self.assertEqual(person.cats.is_resolved(), True)
-
-        person = mock.Person.get(
-            identifier = 1,
-            map = True,
-            eager = ("cats",)
-        )
-
-        self.assertEqual(isinstance(person, dict), True)
-        self.assertEqual(isinstance(person["cats"], list), True)
-        self.assertEqual(isinstance(person["cats"][0], dict), True)
-
-        person = mock.Person.get(identifier = 1)
-
-        person.cats = []
-        person.save()
-
-        person = mock.Person.get(identifier = 1)
-
-        self.assertEqual(len(person.cats), 0)
-
-        person = mock.Person.get(map = True, eager = ("cats",))
-
-        self.assertEqual(isinstance(person, dict), True)
-        self.assertEqual(isinstance(person["cats"], list), True)
-        self.assertEqual(len(person["cats"]), 0)
-
     @quorum.secured
     def test_increment(self):
         person = mock.Person()
@@ -319,6 +258,44 @@ class ModelTest(quorum.TestCase):
         self.assertEqual(person.identifier, 4)
         self.assertEqual(person.name, "Name4")
 
+    def test_ensure_min(self):
+        person = mock.Person()
+        person.identifier = 10
+        person.name = "Name1"
+        person.save()
+
+        self.assertEqual(person.identifier, 10)
+        self.assertEqual(person.name, "Name1")
+
+        person = mock.Person()
+        person.name = "Name2"
+        person.save()
+
+        self.assertEqual(person.identifier, 11)
+        self.assertEqual(person.name, "Name2")
+
+        person = mock.Person()
+        person.name = "Name3"
+        person.save()
+
+        self.assertEqual(person.identifier, 12)
+        self.assertEqual(person.name, "Name3")
+
+        person = mock.Person()
+        person.identifier = 30
+        person.name = "Name4"
+        person.save()
+
+        self.assertEqual(person.identifier, 30)
+        self.assertEqual(person.name, "Name4")
+
+        person = mock.Person()
+        person.name = "Name5"
+        person.save()
+
+        self.assertEqual(person.identifier, 31)
+        self.assertEqual(person.name, "Name5")
+
     @quorum.secured
     def test_sort(self):
         person = mock.Person()
@@ -365,6 +342,108 @@ class ModelTest(quorum.TestCase):
         self.assertEqual(len(result), 7)
         self.assertEqual(result[0].name, "Name3")
 
+    @quorum.secured
+    def test_references(self):
+        person = mock.Person()
+        person.name = "Name"
+
+        cat = mock.Cat()
+        cat.name = "NameCat"
+        cat.save()
+
+        cat_friend = mock.Cat()
+        cat_friend.name = "NameCatFriend"
+        cat_friend.save()
+
+        cat.friend = cat_friend
+        cat.save()
+
+        person.cats = [cat]
+        person.save()
+
+        person = mock.Person.get(identifier = 1)
+
+        self.assertEqual(person.cats[0].name, "NameCat")
+
+        person.cats = mock.Person.cats["type"]([cat])
+        person.save()
+
+        person = mock.Person.get(identifier = 1)
+
+        self.assertEqual(person.cats.is_resolved(), False)
+        self.assertEqual(person.car, None)
+        self.assertEqual(person.cats[0].name, "NameCat")
+
+        person = mock.Person.get(identifier = 1, map = True)
+
+        self.assertEqual(isinstance(person, dict), True)
+        self.assertEqual(isinstance(person["cats"], list), True)
+        self.assertEqual(isinstance(person["cats"][0], int), True)
+        self.assertEqual(len(person["cats"]), 1)
+
+        person = mock.Person.get(identifier = 1, eager = ("cats",))
+
+        self.assertEqual(isinstance(person.cats, quorum.References), True)
+        self.assertEqual(isinstance(person.cats[0].friend, quorum.Reference), True)
+        self.assertEqual(person.cats.is_resolved(), True)
+        self.assertEqual(person.cats[0].friend.is_resolved(), False)
+        self.assertEqual(len(person.cats), 1)
+        self.assertEqual(person.cats[0].name, "NameCat")
+
+        person = mock.Person.get(
+            identifier = 1,
+            map = True,
+            eager = ("cats",)
+        )
+
+        self.assertEqual(isinstance(person, dict), True)
+        self.assertEqual(isinstance(person["cats"], list), True)
+        self.assertEqual(isinstance(person["cats"][0], dict), True)
+        self.assertEqual(isinstance(person["cats"], quorum.References), False)
+        self.assertEqual(isinstance(person["cats"][0]["friend"], quorum.Reference), False)
+        self.assertEqual(len(person["cats"]), 1)
+        self.assertEqual(person["cats"][0]["name"], "NameCat")
+
+        person = mock.Person.get(identifier = 1, eager = ("cats.friend",))
+
+        self.assertEqual(isinstance(person.cats, quorum.References), True)
+        self.assertEqual(isinstance(person.cats[0].friend, quorum.Reference), True)
+        self.assertEqual(person.cats.is_resolved(), True)
+        self.assertEqual(person.cats[0].friend.is_resolved(), True)
+        self.assertEqual(len(person.cats), 1)
+        self.assertEqual(person.cats[0].name, "NameCat")
+        self.assertEqual(person.cats[0].friend.name, "NameCatFriend")
+
+        person = mock.Person.get(
+            identifier = 1,
+            map = True,
+            eager = ("cats.friend",)
+        )
+
+        self.assertEqual(isinstance(person, dict), True)
+        self.assertEqual(isinstance(person["cats"], list), True)
+        self.assertEqual(isinstance(person["cats"][0], dict), True)
+        self.assertEqual(isinstance(person["cats"][0]["friend"], dict), True)
+        self.assertEqual(isinstance(person["cats"], quorum.References), False)
+        self.assertEqual(isinstance(person["cats"][0]["friend"], quorum.Reference), False)
+        self.assertEqual(person["cats"][0]["name"], "NameCat")
+        self.assertEqual(person["cats"][0]["friend"]["name"], "NameCatFriend")
+
+        person = mock.Person.get(identifier = 1)
+
+        person.cats = []
+        person.save()
+
+        person = mock.Person.get(identifier = 1)
+
+        self.assertEqual(len(person.cats), 0)
+
+        person = mock.Person.get(map = True, eager = ("cats",))
+
+        self.assertEqual(isinstance(person, dict), True)
+        self.assertEqual(isinstance(person["cats"], list), True)
+        self.assertEqual(len(person["cats"]), 0)
+
     @quorum.secured
     def test_eager(self):
         person = mock.Person()
@@ -472,6 +551,37 @@ class ModelTest(quorum.TestCase):
         self.assertEqual(person.father.car.is_resolved(), True)
         self.assertEqual(person.father.car.name, "CarFather")
 
+    @quorum.secured
+    def test_unresolvable(self):
+        person = mock.Person()
+        person.name = "Name"
+        person.save()
+
+        car = mock.Car()
+        car.name = "Car"
+        car.save()
+
+        person = mock.Person.get(identifier = 1)
+        person.car = car
+        person.save()
+
+        self.assertEqual(isinstance(person.car, quorum.Reference), False)
+
+        person = mock.Person.get(identifier = 1)
+
+        self.assertEqual(isinstance(person.car, quorum.Reference), True)
+        self.assertEqual(person.car.is_resolvable(), True)
+        self.assertEqual(person.car == None, False)
+        self.assertEqual(person.car.name, "Car")
+
+        car.delete()
+
+        person = mock.Person.get(identifier = 1)
+
+        self.assertEqual(isinstance(person.car, quorum.Reference), True)
+        self.assertEqual(person.car.is_resolvable(), False)
+        self.assertEqual(person.car == None, False)
+
     @quorum.secured
     def test_exists(self):
         person = mock.Person()
@@ -491,6 +601,15 @@ class ModelTest(quorum.TestCase):
 
         self.assertEqual(person.exists(), False)
 
+    @quorum.secured
+    def test_fill(self):
+        first = mock.Person(fill = True)
+        second = mock.Person(fill = True)
+
+        self.assertEqual(first.info, {})
+        self.assertEqual(second.info, {})
+        self.assertNotEqual(id(first.info), id(second.info))
+
     @quorum.secured
     def test_wrap(self):
         person = mock.Person.wrap(dict(name = "Person"))
@@ -506,3 +625,69 @@ class ModelTest(quorum.TestCase):
         self.assertEqual(quorum.Model._to_meta("text"), "text")
         self.assertEqual(quorum.Model._to_meta("longtext"), "longtext")
         self.assertEqual(quorum.Model._to_meta(mock.Person.father["type"]), "reference")
+
+    @quorum.secured
+    def test_meta_map(self):
+        method = quorum.model.METAS["map"]
+
+        map = dict(hello = "world")
+        result = method(map, {})
+
+        self.assertEqual(type(result), str)
+        self.assertEqual(result, "{\"hello\": \"world\"}")
+
+        map = dict(mundo = "olá")
+
+        self.assertEqual(type(result), str)
+        self.assertEqual(method(map, {}), "{\"mundo\": \"olá\"}")
+
+    @quorum.secured
+    def test_meta_longmap(self):
+        method = quorum.model.METAS["longmap"]
+
+        map = dict(hello = "world")
+        result = method(map, {})
+
+        self.assertEqual(type(result), str)
+        self.assertEqual(result, "{\"hello\": \"world\"}")
+
+        map = dict(mundo = "olá")
+
+        self.assertEqual(type(result), str)
+        self.assertEqual(method(map, {}), "{\"mundo\": \"olá\"}")
+
+    @quorum.secured
+    def test_is_unset(self):
+        person = mock.Person()
+        person.name = "Name"
+        person.save()
+
+        father = mock.Person()
+        father.name = "Father"
+        father.save()
+
+        person.father = father
+        person.save()
+
+        self.assertEqual(quorum.is_unset(person.father), False)
+        self.assertEqual(isinstance(person.father, quorum.Reference), False)
+        self.assertEqual(quorum.is_unset(person.car), True)
+        self.assertEqual(person.car, None)
+
+        person = person.reload()
+
+        self.assertEqual(quorum.is_unset(person.father), False)
+        self.assertEqual(isinstance(person.father, quorum.Reference), True)
+        self.assertEqual(person.father.is_resolved(), True)
+        self.assertEqual(quorum.is_unset(person.car), True)
+        self.assertEqual(person.car, None)
+
+        father.delete()
+
+        person = person.reload()
+
+        self.assertEqual(quorum.is_unset(person.father), True)
+        self.assertEqual(isinstance(person.father, quorum.Reference), True)
+        self.assertEqual(person.father.is_resolved(), False)
+        self.assertEqual(quorum.is_unset(person.car), True)
+        self.assertEqual(person.car, None)
