@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Flask Quorum
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2020 Hive Solutions Lda.
 #
 # This file is part of Hive Flask Quorum.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2020 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -47,8 +47,9 @@ from . import legacy
 from . import common
 from . import config
 from . import storage
+from . import exceptions
 
-class Type(object):
+class AbstractType(object):
 
     def json_v(self, *args, **kwargs):
         return str(self)
@@ -56,7 +57,22 @@ class Type(object):
     def map_v(self, *args, **kwargs):
         return self.json_v()
 
-class File(Type):
+class Type(AbstractType):
+
+    def __init__(self, value):
+        cls = self.__class__
+        self.loads(value)
+
+    def json_v(self, *args, **kwargs):
+        return self.dumps()
+
+    def loads(self, value):
+        raise exceptions.NotImplementedError()
+
+    def dumps(self):
+        raise exceptions.NotImplementedError()
+
+class File(AbstractType):
 
     def __init__(self, file):
         if isinstance(file, legacy.BYTES): self.build_d(file)
@@ -254,7 +270,7 @@ class File(Type):
         if not self.engine: return storage.BaseEngine
         return getattr(storage, self.engine.capitalize() + "Engine")
 
-class Files(Type):
+class Files(AbstractType):
 
     def __init__(self, files):
         if isinstance(files, Files): self.build_i(files)
@@ -303,6 +319,7 @@ class ImageFile(File):
         self.width = file_m.get("width", 0)
         self.height = file_m.get("height", 0)
         self.format = file_m.get("format", None)
+        self.kwargs = file_m.get("kwargs", {})
         self._ensure_all()
 
     def build_t(self, file_t):
@@ -313,7 +330,8 @@ class ImageFile(File):
         File.build_i(self, file)
         self.width = file.width if hasattr(file, "width") else 0
         self.height = file.height if hasattr(file, "height") else 0
-        self.format = file.height if hasattr(file, "format") else None
+        self.format = file.format if hasattr(file, "format") else None
+        self.kwargs = file.kwargs if hasattr(file, "kwargs") else {}
         self._ensure_all()
 
     def build_f(self, file):
@@ -326,29 +344,40 @@ class ImageFile(File):
         value.update(
             width = self.width,
             height = self.height,
-            format = self.format
+            format = self.format,
+            kwargs = self.kwargs
         )
         return value
 
     def _ensure_all(self):
         self._ensure_size()
         self._ensure_mime()
+        self._ensure_kwargs()
 
     def _ensure_size(self):
-        if self.width and self.height: return
+        if hasattr(self, "width") and self.width and\
+            hasattr(self, "height") and self.height: return
         self.width, self.height = self._size()
 
     def _ensure_mime(self):
-        if self.format and self.mime: return
+        if hasattr(self, "format") and self.format and\
+            hasattr(self, "mime") and self.mime: return
         self.format, self.mime = self._mime()
 
+    def _ensure_kwargs(self):
+        if hasattr(self, "kwargs"): return
+        self.kwargs = dict()
+
     def _size(self):
         try: return self._size_image()
-        except: return self._size_default()
+        except Exception: return self._size_default()
 
     def _size_image(self):
+        if self.data: return self._size_pil()
+        else: return self._size_default()
+
+    def _size_pil(self):
         import PIL.Image
-        if not self.data: return self._size_default()
         buffer = legacy.BytesIO(self.data)
         try:
             image = PIL.Image.open(buffer)
@@ -362,11 +391,14 @@ class ImageFile(File):
 
     def _mime(self):
         try: return self._mime_image()
-        except: return self._mime_default()
+        except Exception: return self._mime_default()
 
     def _mime_image(self):
+        if self.data: return self._mime_pil()
+        else: return self._mime_default()
+
+    def _mime_pil(self):
         import PIL.Image
-        if not self.data: return self._size_default()
         buffer = legacy.BytesIO(self.data)
         try:
             image = PIL.Image.open(buffer)
@@ -384,20 +416,36 @@ class ImageFiles(Files):
     def base(self):
         return ImageFile
 
-def image(width = None, height = None, format = "png"):
+def image(width = None, height = None, format = "png", **kwargs):
 
     class _ImageFile(ImageFile):
 
+        RGB_FORMATS = ("jpg", "jpeg")
+        """ The set of format that are considered to be
+        RGB based and should be handled with special care """
+
         def build_b64(self, file_m):
             ImageFile.build_b64(self, file_m)
 
+            # verifies if there's valid data in the current file
+            # if that's not the case there's nothing remaining to
+            # be done (not going to process the file)
+            if not self.data_b64: return
+
+            # determines if a resize operation is required for the
+            # current image data, if that's not the case returns the
+            # control flow immediately (nothing to be done)
             need_resize = self.need_resize(
                 width_o = self.width,
                 height_o = self.height,
-                format_o = self.format
+                format_o = self.format,
+                kwargs_o = self.kwargs
             )
             if not need_resize: return
 
+            # decodes the base 64 based data and then runs the resize
+            # operation with the current data re-encoding it back to
+            # the base 64 model to be used in the map storage
             data_b64 = legacy.bytes(self.data_b64)
             data = base64.b64decode(data_b64)
             data = self.resize(data)
@@ -410,37 +458,60 @@ def image(width = None, height = None, format = "png"):
             # width, height and format from the original one
             file_m["data"] = data_b64
 
+            # updates the parameters attributes in the file map so
+            # that the new file is marked with proper values
+            file_m["kwargs"] = kwargs
+
             # removes a series of keys from the file map as
             # they are no longer reliable and may reflect
             # different values from the ones contained in data
-            # theses should be update b y the "ensure" methods
+            # theses should be update by the "ensure" methods
             for name in ("width", "height", "format"):
                 if name in file_m: del file_m[name]
 
+            # runs the rebuilding of the information taking into
+            # account the new information from the file
             ImageFile.build_b64(self, file_m)
 
         def build_t(self, file_t):
+            # unpacks the file tuple into its component to be
+            # used for the format and size ensure operations
             name, content_type, data = file_t
-            try: _data = self.resize(data)
-            except: _data = data
+
+            # tries to run the resize operation to ensure that
+            # the proper size and format is present in the data
+            try: _data = self.resize(data) if data else data
+            except Exception: _data = data
+
+            # updates the parameters attributes in the instance so
+            # that the new file is marked with proper values
+            self.kwargs = kwargs
+
+            # creates the "new" file tuple with the newly resized
+            # image data and runs the parent tuple build method
             file_t = (name, content_type, _data)
             ImageFile.build_t(self, file_t)
 
         def resize(self, data = None):
             import PIL.Image
+
             data = data or self.data
             if not data: return data
 
             is_resized = True if width or height else False
             if not is_resized: return data
 
+            params = kwargs.get("params", {})
+            background = kwargs.get("background", "ffffff")
+
             size = (width, height)
             in_buffer = legacy.BytesIO(data)
             out_buffer = legacy.BytesIO()
             try:
                 image = PIL.Image.open(in_buffer)
                 image = self._resize(image, size)
-                image.save(out_buffer, format)
+                image = self._format(image, format, background)
+                image.save(out_buffer, format, **params)
                 data = out_buffer.getvalue()
             finally:
                 in_buffer.close()
@@ -452,11 +523,13 @@ def image(width = None, height = None, format = "png"):
             self,
             width_o = None,
             height_o = None,
-            format_o = None
+            format_o = None,
+            kwargs_o = None
         ):
             if width and not width == width_o: return True
             if height and not height == height_o: return True
             if format and not format == format_o: return True
+            if kwargs and not kwargs == kwargs_o: return True
             return False
 
         def _resize(self, image, size):
@@ -508,14 +581,39 @@ def image(width = None, height = None, format = "png"):
             image = image.resize(size, PIL.Image.ANTIALIAS)
             return image
 
+        def _format(self, image, format, background):
+            import PIL.Image
+
+            # retrieves the reference to the top level class that is
+            # going to be used for the access to some of the values
+            cls = self.__class__
+
+            # converts the target format into the lower based version
+            # (normalization) and verifies if it's considered to be
+            # an RGB only format, if that's not the case there's nothing
+            # remaining to be done (no conversion required)
+            format = format.lower()
+            if not format in cls.RGB_FORMATS: return image
+
+            # in case the format/mode of the original image is already
+            # RGB there's no need to run the conversion process
+            if image.mode == "RGB": return image
+
+            # creates a new RGB based image with the target size and
+            # with the provided background and copies it as the target
+            new = PIL.Image.new("RGB", (image.width, image.height), "#" + background)
+            new.paste(image, image)
+            return new
+
     return _ImageFile
 
-def images(width = None, height = None, format = "png"):
+def images(width = None, height = None, format = "png", **kwargs):
 
     image_c = image(
         width = width,
         height = height,
-        format = format
+        format = format,
+        **kwargs
     )
 
     class _ImageFiles(ImageFiles):
@@ -525,7 +623,7 @@ def images(width = None, height = None, format = "png"):
 
     return _ImageFiles
 
-class Reference(Type):
+class Reference(AbstractType):
     pass
 
 def reference(target, name = None, dumpall = False):
@@ -602,6 +700,7 @@ def reference(target, name = None, dumpall = False):
         def __start__(self):
             if is_reference: self._target = self.__class__._target()
             else: self._target = target
+            util.verify(self._target)
             meta = getattr(self._target, name)
             self._type = meta.get("type", legacy.UNICODE)
 
@@ -672,7 +771,7 @@ def reference(target, name = None, dumpall = False):
                 self.__dict__["_object"] = _object
                 return _object
 
-            # creates the map of keyword based arguments that is going
+            # creates the map of keyword based arguments that are going
             # to be used in the resolution of the reference and uses the
             # data source based get attribute to retrieve the object
             # that represents the reference
@@ -680,6 +779,7 @@ def reference(target, name = None, dumpall = False):
             kwargs[name] = self._target.cast(name, self.id)
             kwargs["raise_e"] = kwargs.get("raise_e", False)
             kwargs["eager_l"] = kwargs.get("eager_l", False)
+            kwargs["resolve_a"] = kwargs.get("resolve_a", False)
             _object = self._target.get(*args, **kwargs)
 
             # sets the resolved object (using the current id attribute)
@@ -698,13 +798,19 @@ def reference(target, name = None, dumpall = False):
             exists = "_object" in self.__dict__
             return True if exists and self._object else False
 
+        def is_resolvable(self):
+            self.resolve()
+            return False if self._object == None else True
+
     return _Reference
 
-class References(Type):
+class References(AbstractType):
     pass
 
 def references(target, name = None, dumpall = False):
     name = name or "id"
+    target_t = type(target)
+    is_reference = target_t in legacy.STRINGS
     reference_c = reference(target, name = name, dumpall = dumpall)
 
     class _References(References):
@@ -715,6 +821,7 @@ def references(target, name = None, dumpall = False):
         may latter be used to cast the value """
 
         def __init__(self, ids):
+            self.__start__()
             if isinstance(ids, _References): self.build_i(ids)
             else: self.build(ids)
 
@@ -733,6 +840,11 @@ def references(target, name = None, dumpall = False):
         def __contains__(self, item):
             return self.contains(item)
 
+        def __start__(self):
+            if is_reference: self._target = self.__class__._target()
+            else: self._target = target
+            util.verify(self._target)
+
         @classmethod
         def _default(cls):
             return cls([])
@@ -787,6 +899,16 @@ def references(target, name = None, dumpall = False):
         def resolve(self, *args, **kwargs):
             return [object.resolve(*args, **kwargs) for object in self.objects]
 
+        def find(self, *args, **kwargs):
+            kwargs = dict(kwargs)
+            kwargs[name] = {"$in" : [self._target.cast(name, _id) for _id in self.ids]}
+            return self._target.find(*args, **kwargs)
+
+        def paginate(self, *args, **kwargs):
+            kwargs = dict(kwargs)
+            kwargs[name] = {"$in" : [self._target.cast(name, _id) for _id in self.ids]}
+            return self._target.paginate(*args, **kwargs)
+
         def is_empty(self):
             ids_l = len(self.ids)
             return ids_l == 0
@@ -818,7 +940,7 @@ def references(target, name = None, dumpall = False):
 
     return _References
 
-class Encrypted(Type):
+class Encrypted(AbstractType):
 
     PADDING = ":encrypted"
 
