@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Flask Quorum
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2020 Hive Solutions Lda.
 #
 # This file is part of Hive Flask Quorum.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2020 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -43,34 +43,34 @@ import flask
 import datetime
 
 from . import util
-from . import legacy
 from . import common
+from . import legacy
 from . import exceptions
 
 SIMPLE_REGEX_VALUE = "^[\:\.\s\w-]+$"
 """ The simple regex value used to validate
 if the provided value is a "simple" one meaning
-that it may be used safely for url parts """
+that it may be used safely for URL parts """
 
 EMAIL_REGEX_VALUE = "^[\w\d\._%+-]+@[\w\d\.\-]+$"
 """ The email regex value used to validate
 if the provided value is in fact an email """
 
 URL_REGEX_VALUE = "^\w+\:\/\/[^\:\/\?#]+(\:\d+)?(\/[^\?#]+)*\/?(\?[^#]*)?(#.*)?$"
-""" The url regex value used to validate
+""" The URL regex value used to validate
 if the provided value is in fact an URL/URI """
 
 SIMPLE_REGEX = re.compile(SIMPLE_REGEX_VALUE)
 """ The simple regex used to validate
 if the provided value is a "simple" one meaning
-that it may be used safely for url parts """
+that it may be used safely for URL parts """
 
 EMAIL_REGEX = re.compile(EMAIL_REGEX_VALUE)
 """ The email regex used to validate
 if the provided value is in fact an email """
 
 URL_REGEX = re.compile(URL_REGEX_VALUE)
-""" The url regex used to validate
+""" The URL regex used to validate
 if the provided value is in fact an URL/URI """
 
 def validate(method = None, methods = [], object = None, ctx = None, build = True):
@@ -89,7 +89,7 @@ def validate(method = None, methods = [], object = None, ctx = None, build = Tru
     # to correctly retrieve populate the object from it
     if build:
         # retrieves the current request data and tries to
-        # "load" it as json data, in case it fails gracefully
+        # "load" it as JSON data, in case it fails gracefully
         # handles the failure setting the value as an empty map
         data_j = util.request_json()
 
@@ -229,6 +229,16 @@ def is_in(name, values, message = "value is not in set", locale = True):
         raise exceptions.ValidationInternalError(name, message_l)
     return validation
 
+def is_upper(name, message = "value contains lower cased characters", locale = True):
+    def validation(object, ctx):
+        value = object.get(name, None)
+        if value == None: return True
+        if value == "": return True
+        if value == value.upper(): return True
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l)
+    return validation
+
 def is_lower(name, message = "value contains upper cased characters", locale = True):
     def validation(object, ctx):
         value = object.get(name, None)
@@ -259,7 +269,7 @@ def is_email(name, message = "value is not a valid email", locale = True):
         raise exceptions.ValidationInternalError(name, message_l)
     return validation
 
-def is_url(name, message = "value is not a valid url", locale = True):
+def is_url(name, message = "value is not a valid URL", locale = True):
     def validation(object, ctx):
         value = object.get(name, None)
         if value == None: return True
